----------------------------------------------------------- 
-- Video frame grabber test bench 
-- 
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use std.textio.all;

use work.all;

entity T_CONTROL is
end entity T_CONTROL;

architecture stimulus of T_CONTROL is
  component CONTROL is
    port (
      Reset    : in  std_logic;
      Clk      : in  std_logic;
      Mode     : in  std_logic;
      Data     : in  std_logic_vector(7 downto 0);
      TestLoad : in  std_logic;
      Addr     : out std_logic_vector(17 downto 0);
      RAMWE    : out std_logic;
      RAMOE    : out std_logic;
      ADOE     : out std_logic
      ); 
  end component CONTROL;
  constant PERIOD   : time := 100 ns;
  -- Top level signals go here... 
  signal   Reset    : std_logic;
  signal   Clk      : std_logic;
  signal   Mode     : std_logic;
  signal   Data     : std_logic_vector(7 downto 0);
  signal   TestLoad : std_logic;
  signal   Addr     : std_logic_vector(17 downto 0);
  signal   RAMWE    : std_logic;
  signal   RAMOE    : std_logic;
  signal   ADOE     : std_logic;
  signal   done     : boolean := false;
  
begin
  DUT : CONTROL port map (
    Reset    => Reset,
    Clk      => Clk,
    Mode     => Mode,
    Data     => Data,
    TestLoad => TestLoad,
    Addr     => Addr,
    RAMWE    => RAMWE,
    RAMOE    => RAMOE,
    ADOE     => ADOE
    ); 

  Clock1 : process
    variable clktmp : std_logic := '0';
  begin
    wait for PERIOD/2;
    clktmp := not clktmp;
    Clk    <= clktmp;                   -- Attach your clock here

    if done = true then
      wait;
    end if;
  end process Clock1;

  Stimulus1 : process
  begin
    -- Sequential stimulus goes here... 
    Reset    <= '1';
    Mode     <= '0';
    Data     <= "00000000";
    TestLoad <= '0';
    wait for PERIOD;
    Reset    <= '0';
    wait for PERIOD;
    Data     <= "00000001";
    wait for PERIOD;
    Mode     <= '1';

    -- Check to make sure we detect the vertical sync... 
    Data <= "00000000";
    for i in 0 to 127 loop
      wait for PERIOD;
    end loop;

    -- Now sample data to make sure the frame counter works... 
    Data <= "01010101";
    for i in 0 to 100000 loop
      wait for PERIOD;
    end loop;

    -- Load in the test value to check the end of frame detection... 
    TestLoad <= '1';
    wait for PERIOD;
    TestLoad <= '0';
    for i in 0 to 300 loop
      wait for PERIOD;
    end loop;
    done <= true;
  end process Stimulus1;
end architecture stimulus;
